@@ -22,14 +22,6 @@
 # "fulltest"). The program may be invoked as follows:
 #
 set USAGE {
-$argv0 platforms
-    List available platforms.
-
-$argv0 tests ?-nodebug? PLATFORM
-    List tests in a specified platform. If the -nodebug switch is 
-    specified, synthetic debug/ndebug configurations are omitted. Each
-    test is a combination of a configuration and a makefile target.
-
 $argv0 script ?-msvc? CONFIGURATION TARGET
     Given a configuration and make target, return a bash (or, if -msvc
     is specified, batch) script to execute the test. The first argument
@@ -37,6 +29,14 @@ $argv0 script ?-msvc? CONFIGURATION TARGET
 
 $argv0 configurations
     List available configurations.
+
+$argv0 platforms
+    List available platforms.
+
+$argv0 tests ?-nodebug? PLATFORM
+    List tests in a specified platform. If the -nodebug switch is 
+    specified, synthetic debug/ndebug configurations are omitted. Each
+    test is a combination of a configuration and a makefile target.
 }
 
 # Omit comments (text between # and \n) in a long multi-line string.
@@ -51,11 +51,11 @@ array set ::Configs [strip_comments {
     -O2
     --disable-amalgamation --disable-shared
     --enable-session
-    -DSQLITE_ENABLE_DESERIALIZE
   }
   "Sanitize" {
     CC=clang -fsanitize=address,undefined
     -DSQLITE_ENABLE_STAT4
+    -DCONFIG_SLOWDOWN_FACTOR=5.0
     --enable-debug
     --enable-all
   }
@@ -149,6 +149,7 @@ array set ::Configs [strip_comments {
     -DSQLITE_ENABLE_RBU
     -DSQLITE_MAX_ATTACHED=125
     -DSQLITE_MAX_MMAP_SIZE=12884901888
+    -DSQLITE_ENABLE_SORTER_MMAP=1
     -DLONGDOUBLE_TYPE=double
     --enable-session
   }
@@ -187,7 +188,6 @@ array set ::Configs [strip_comments {
     -DSQLITE_OMIT_TRACE=1
     -DSQLITE_TEMP_STORE=3
     -DSQLITE_THREADSAFE=2
-    -DSQLITE_ENABLE_DESERIALIZE=1
     --enable-json1 --enable-fts5 --enable-session
   }
   "Locking-Style" {
@@ -258,6 +258,7 @@ array set ::Configs [strip_comments {
     -DSQLITE_ENABLE_FTS4
     -DSQLITE_ENABLE_RTREE
     -DSQLITE_ENABLE_HIDDEN_COLUMNS
+    -DCONFIG_SLOWDOWN_FACTOR=8.0
     --enable-json1
   }
 
@@ -283,61 +284,66 @@ array set ::Configs [strip_comments {
   FuzzFail2 {-O0}
 }]
 if {$tcl_platform(os)=="Darwin"} {
-  lappend Configs(Apple -DSQLITE_ENABLE_LOCKING_STYLE=1
+  lappend Configs(Apple) -DSQLITE_ENABLE_LOCKING_STYLE=1
 }
 
 array set ::Platforms [strip_comments {
   Linux-x86_64 {
-    "Check-Symbols*"          checksymbols
-    "Fast-One"                "fuzztest test"
-    "Debug-One"               "mptest test"
-    "Debug-Two"               "test"
-    "Have-Not"                test
-    "Secure-Delete"           test
-    "Unlock-Notify"           "QUICKTEST_INCLUDE=notify2.test test"
-    "User-Auth"               tcltest
-    "Update-Delete-Limit"     test
-    "Extra-Robustness"        test
-    "Device-Two"              "threadtest test"
-    "No-lookaside"            test
-    "Devkit"                  test
-    "Apple"                   test
-    "Sanitize"                {QUICKTEST_OMIT=crash*,shell*,sqldiff*,sessionB.test test}
-    "Device-One"              fulltest
-    "Default"                 "threadtest fulltest"
-    "Valgrind*"               valgrindtest
+    "Check-Symbols*"          "" checksymbols
+    "Fast-One"                QUICKTEST_INCLUDE=rbu.test "fuzztest test"
+    "Debug-One"               "" "mptest test"
+    "Debug-Two"               "" test
+    "Have-Not"                "" test
+    "Secure-Delete"           "" test
+    "Unlock-Notify"           QUICKTEST_INCLUDE=notify2.test test
+    "User-Auth"               "" tcltest
+    "Update-Delete-Limit"     "" test
+    "Extra-Robustness"        "" test
+    "Device-Two"              "" "threadtest test"
+    "No-lookaside"            "" test
+    "Devkit"                  "" test
+    "Apple"                   "" test
+    "Sanitize*"               "" test
+    "Device-One"              "" alltest
+    "Default"                 "" "threadtest fuzztest alltest"
+    "Valgrind*"               "" valgrindtest
   }
   Linux-i686 {
-    "Devkit"                  test
-    "Have-Not"                test
-    "Unlock-Notify"           "QUICKTEST_INCLUDE=notify2.test test"
-    "Device-One"              test
-    "Device-Two"              test
-    "Default"                 "threadtest fulltest"
+    "Devkit"                  "" test
+    "Have-Not"                "" test
+    "Unlock-Notify"           QUICKTEST_INCLUDE=notify2.test test
+    "Device-One"              "" test
+    "Device-Two"              "" test
+    "Default"                 "" "threadtest fuzztest alltest"
   }
   Darwin-i386 {
-    "Locking-Style"           "mptest test"
-    "Have-Not"                test
-    "Apple"                   "threadtest fulltest"
+    "Locking-Style"           "" "mptest test"
+    "Have-Not"                "" test
+    "Apple"                   "" "threadtest fuzztest alltest"
   }
   Darwin-x86_64 {
-    "Locking-Style"           "mptest test"
-    "Have-Not"                test
-    "Apple"                   "threadtest fulltest"
+    "Locking-Style"           "" "mptest test"
+    "Have-Not"                "" test
+    "Apple"                   "" "threadtest fuzztest alltest"
+  }
+  Darwin-arm64 {
+    "Locking-Style"           "" "mptest test"
+    "Have-Not"                "" test
+    "Apple"                   "" "threadtest fuzztest alltest"
   }
   "Windows NT-intel" {
-    "Stdcall"                 test
-    "Have-Not"                test
-    "Windows-Memdebug*"       test
-    "Windows-Win32Heap*"      test
-    "Default"                 "mptest fulltestonly"
+    "Stdcall"                 "" test
+    "Have-Not"                "" test
+    "Windows-Memdebug*"       "" test
+    "Windows-Win32Heap*"      "" test
+    "Default"                 "" "mptest fulltestonly"
   }
   "Windows NT-amd64" {
-    "Stdcall"                 test
-    "Have-Not"                test
-    "Windows-Memdebug*"       test
-    "Windows-Win32Heap*"      test
-    "Default"                 "mptest fulltestonly"
+    "Stdcall"                 "" test
+    "Have-Not"                "" test
+    "Windows-Memdebug*"       "" test
+    "Windows-Win32Heap*"      "" test
+    "Default"                 "" "mptest fulltestonly"
   }
 
   # The Failure-Detection platform runs various tests that deliberately
@@ -345,21 +351,29 @@ array set ::Platforms [strip_comments {
   # correctly identifies failures.
   #
   Failure-Detection {
-    Fail0*     "TEST_FAILURE=0 test"
-    Sanitize*  "TEST_FAILURE=1 test"
-    Fail2*     "TEST_FAILURE=2 valgrindtest"
-    Fail3*     "TEST_FAILURE=3 valgrindtest"
-    Fail4*     "TEST_FAILURE=4 test"
-    FuzzFail1* "TEST_FAILURE=5 test"
-    FuzzFail2* "TEST_FAILURE=5 valgrindtest"
+    Fail0*     "TEST_FAILURE=0" test
+    Sanitize*  "TEST_FAILURE=1" test
+    Fail2*     "TEST_FAILURE=2" valgrindtest
+    Fail3*     "TEST_FAILURE=3" valgrindtest
+    Fail4*     "TEST_FAILURE=4" test
+    FuzzFail1* "TEST_FAILURE=5" test
+    FuzzFail2* "TEST_FAILURE=5" valgrindtest
   }
 }]
 
+#--------------------------------------------------------------------------
+#--------------------------------------------------------------------------
+#--------------------------------------------------------------------------
+# End of configuration section.
+#--------------------------------------------------------------------------
+#--------------------------------------------------------------------------
+#--------------------------------------------------------------------------
+
 # Configuration verification: Check that each entry in the list of configs
 # specified for each platforms exists.
 #
 foreach {key value} [array get ::Platforms] {
-  foreach {v t} $value {
+  foreach {v vars t} $value {
     if {[string range $v end end]=="*"} {
       set v [string range $v 0 end-1]
     }
@@ -430,6 +444,7 @@ proc main_script {args} {
     lappend opts -DSQLITE_EXTRA_IFNULLROW
     set config [string range $config 0 end-6]
   }
+  regexp {^(.*)-[0-9]+} $config -> config
 
   # Ensure that the named configuration exists.
   #
@@ -465,6 +480,7 @@ proc main_script {args} {
     if {$bRemoveDebug} {
       if {$param=="-DSQLITE_DEBUG" || $param=="-DSQLITE_DEBUG=1"
        || $param=="-DSQLITE_MEMDEBUG" || $param=="-DSQLITE_MEMDEBUG=1"
+       || $param=="--enable-debug"
       } {
         continue
       }
@@ -577,26 +593,57 @@ proc main_tests {args} {
     exit 1
   }
 
-  foreach {config target} $::Platforms($p) {
-    set bNosynthetic 0
+  set lTest [list]
+
+  foreach {config vars target} $::Platforms($p) {
     if {[string range $config end end]=="*"} {
-      set bNosynthetic 1
       set config [string range $config 0 end-1]
-    }
-    puts "$config \"$target\""
-    if {$bNodebug==0 && $bNosynthetic==0} {
-      set iHas [string first SQLITE_DEBUG $::Configs($config)]
+    } elseif {$bNodebug==0} {
       set dtarget test
-      if {$target=="tcltest"} {
+      if {[lsearch $target fuzztest]<0 && [lsearch $target test]<0} {
         set dtarget tcltest
       }
-      if {$iHas>=0} {
-        puts "$config-ndebug \"$dtarget\""
+      if {$vars!=""} { set dtarget "$vars $dtarget" }
+
+      if {[string first SQLITE_DEBUG $::Configs($config)]>=0
+       || [string first --enable-debug $::Configs($config)]>=0
+      } {
+        lappend lTest "$config-ndebug \"$dtarget\""
       } else {
-        puts "$config-debug \"$dtarget\""
+        lappend lTest "$config-debug \"$dtarget\""
+      }
+    }
+
+    if {[llength $target]==1 && ([string match "*TEST_FAILURE*" $vars] || (
+        [lsearch $target "valgrindtest"]<0
+     && [lsearch $target "alltest"]<0
+     && [lsearch $target "fulltestonly"]<0
+     && ![string match Sanitize* $config]
+    ))} {
+      if {$vars!=""} { set target "$vars $target" }
+      lappend lTest "$config \"$target\""
+    } else {
+      set idir -1
+      foreach t $target {
+        if {$t=="valgrindtest" || $t=="alltest" || $t=="fulltestonly"
+         || [string match Sanitize* $config]
+        } {
+          if {$vars!=""} { set t "$vars $t" }
+          for {set ii 1} {$ii<=4} {incr ii} {
+            lappend lTest "$config-[incr idir] \"TCLTEST_PART=$ii/4 $t\""
+          }
+        } else {
+          if {$vars!=""} { set t "$vars $t" }
+          lappend lTest "$config-[incr idir] \"$t\""
+        }
       }
     }
   }
+
+  foreach l $lTest {
+    puts $l
+  }
+
 }
 
 if {[llength $argv]==0} { usage }
@@ -613,5 +660,3 @@ if {[string match ${cmd}* configurations] && $n==0} {
 } else {
   usage
 }
-
-
