@@ -62,6 +62,9 @@ while test "$1" != ""; do
     --without-rowid)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
         ;;
+    --strict)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
+        ;;
     --nomemstat)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
         ;;
@@ -79,6 +82,10 @@ while test "$1" != ""; do
         ;;
     --legacy)
 	doWal=0
+        CC_OPTS="$CC_OPTS -DSPEEDTEST_OMIT_HASH"
+        ;;
+    --verify)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --verify"
         ;;
     --wal)
         doWal=1
@@ -89,6 +96,9 @@ while test "$1" != ""; do
     --cachesize)
         shift; SPEEDTEST_OPTS="$SPEEDTEST_OPTS --cachesize $1"
         ;;
+    --checkpoint)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --checkpoint"
+        ;;
     --explain)
         doExplain=1
         ;;
